@@ -1,6 +1,6 @@
 /* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
 /*
- *  Copyright (C) 2011-2013  Ruby-GNOME2 Project Team
+ *  Copyright (C) 2011-2015  Ruby-GNOME2 Project Team
  *  Copyright (C) 2002,2003  Masahiro Sakai
  *
  *  This library is free software; you can redistribute it and/or
@@ -39,6 +39,22 @@ rbg_rval2cstr(VALUE *str)
         *str = rb_str_export_to_enc(*str, rb_utf8_encoding());
 #endif
 
+    return StringValueCStr(*str);
+}
+
+const gchar *
+rbg_rval2cstr_raw(VALUE *str)
+{
+    StringValue(*str);
+
+    return StringValueCStr(*str);
+}
+
+const gchar *
+rbg_rval2cstr_ptr(VALUE *str)
+{
+    StringValue(*str);
+
     return RSTRING_PTR(*str);
 }
 
@@ -62,6 +78,18 @@ rbg_rval2cstr_accept_nil(VALUE *str)
     return NIL_P(*str) ? NULL : RVAL2CSTR(*str);
 }
 
+const gchar *
+rbg_rval2cstr_raw_accept_nil(VALUE *str)
+{
+    return NIL_P(*str) ? NULL : RVAL2CSTR_RAW(*str);
+}
+
+const gchar *
+rbg_rval2cstr_ptr_accept_nil(VALUE *str)
+{
+    return NIL_P(*str) ? NULL : RVAL2CSTR_PTR(*str);
+}
+
 /* TODO: How do we deal with encodings? */
 const gchar *
 rbg_rval2cstr_accept_symbol(volatile VALUE *value)
@@ -866,11 +894,17 @@ rbg_scan_options (VALUE options, ...)
     VALUE *value;
     va_list args;
 
-    options = rbg_check_hash_type(options);
     if (NIL_P(options)) {
         options = rb_hash_new();
-    } else if (options == original_options) {
-        options = rb_funcall(options, rb_intern("dup"), 0);
+    } else {
+        options = rbg_check_hash_type(options);
+        if (options == original_options) {
+            options = rb_funcall(options, rb_intern("dup"), 0);
+        } else if (NIL_P(options)) {
+            rb_raise(rb_eArgError,
+                     "options must be Hash or nil: %+" PRIsVALUE,
+                     original_options);
+        }
     }
 
     available_keys = rb_ary_new();
@@ -1155,4 +1189,8 @@ union       GDoubleIEEE754;
     Init_glib_unichar();
     Init_glib_keyfile();
     Init_glib_bookmark_file();
+    Init_glib_variant_type();
+    Init_glib_variant();
+    Init_glib_regex();
+    Init_glib_matchinfo();
 }
